/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

package org.jboss.serial.contribtests.customertest;

import org.jboss.serial.io.JBossObjectOutputStream;
import org.jboss.serial.io.JBossObjectInputStream;

import java.io.*;
import java.util.Random;

/**
 * @author prashantp
 * @version $Revision: 231 $
 */
public class FileSystemResultKeeperDao {

    private static Random random = new Random();

    public File keepResult(Object result, boolean useJBS, boolean useInDirect) {

        FileOutputStream fos = null;
        BufferedOutputStream bufStream = null;
        ObjectOutputStream oos = null;
        try {

            File file = createNewFile();

            fos = new FileOutputStream(file);
            bufStream = new BufferedOutputStream(fos, 200000);
            if (useJBS)
            {
                oos = new JBossObjectOutputStream(bufStream);
            } else
            {
                oos = new ObjectOutputStream(bufStream);
            }

            if (useInDirect)
            {
               ((JBossObjectOutputStream)oos).writeObjectUsingDataContainer(result);
            } else
            {
                oos.writeObject(result);
            }
            oos.flush();
            bufStream.flush();
            fos.flush();
            return file;
        } catch (FileNotFoundException e) {
        	e.printStackTrace();
            throw new RuntimeException("internal.server.error", e);
        } catch (IOException e) {
            e.printStackTrace();
            throw new RuntimeException("internal.server.error", e);
        } finally {
            try {
                oos.close();
                bufStream.close();
                fos.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }


    private File createNewFile() throws IOException {
        return File.createTempFile("serializationTest",".ser");
    }

    public Object getResult(File file, boolean useJBS, boolean useInDirect) {
        FileInputStream fis = null;
        BufferedInputStream bufStream = null;
        ObjectInputStream oos = null;
        try {
            fis = new FileInputStream(file);
            bufStream = new BufferedInputStream(fis, 2000000);
            if (useJBS)
            {
                oos = new JBossObjectInputStream(bufStream);
            } else
            {
                oos = new ObjectInputStream(bufStream);
            }

            Object obj = null;
            if (useInDirect)
            {
                obj = ((JBossObjectInputStream)oos).readObjectUsingDataContainer();
            } else
            {
                obj = oos.readObject();
            }
            return obj;
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            throw new RuntimeException("internal.server.error", e);
        } catch (IOException e) {
            e.printStackTrace();
            throw new RuntimeException("internal.server.error", e);
        } catch (ClassNotFoundException cnfe) {
        	cnfe.printStackTrace();
            throw new RuntimeException("internal.server.error", cnfe);
        } finally {
            try {
                oos.close();
                bufStream.close();
                fis.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

}
