/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

package org.jboss.serial.contribtests.customertest;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 * $Id: ModelServer.java 217 2006-04-18 18:42:42Z csuconic $
 *
 * @author <a href="mailto:clebert.suconic@jboss.com">Clebert Suconic</a>
 */
public class ModelServer {
    private static Random random = new Random();

    public static List getInsuranceModelHundredCustomers() {

        List results = new ArrayList();

        for (int i = 0; i < 100; i++) {
            Customer customer = getCustomer();
            Address address = getAddress();
            customer.setAddress(address);

            for (int j = 0; j < 50; j++)
                customer.addHistory(getCustomerHistory());

            for (int k = 0; k < 50; k++)
                address.addHistory(getAddressHistory());
            customer.setAddress(address);

            for (int l = 0; l < 10; l++) {
                Policy policy = getPolicy();

                for (int m = 0; m < 100; m++) {
                    Claim claim = getClaim();
                    policy.addClaims(claim);
                }
                customer.addPolicy(policy);
            }
            results.add(customer);
        }

        return results;
    }

    protected static List getInsuranceModelFitfyCustomer() {
        List results = new ArrayList();

        for (int i = 0; i < 50; i++) {
            Customer customer = getCustomer();
            Address address = getAddress();
            customer.setAddress(address);

            for (int j = 0; j < 20; j++)
                customer.addHistory(getCustomerHistory());

            for (int k = 0; k < 20; k++)
                address.addHistory(getAddressHistory());
            customer.setAddress(address);

            for (int l = 0; l < 10; l++) {
                Policy policy = getPolicy();

                for (int m = 0; m < 50; m++) {
                    Claim claim = getClaim();
                    policy.addClaims(claim);
                }
                customer.addPolicy(policy);
            }
            results.add(customer);
        }
        return results;
    }

    private static AddressHistory getAddressHistory()
    {
        AddressHistory history = new AddressHistory();
        history.setAddressLine1("sdfklsjdflksjdflskjfd" + random.nextLong());
        history.setAddressLine2("sdlkfjslfdkjslfdkjslfkj"+ random.nextLong());
        history.setAddressLine3("sdklfjsldfkjsdlkfjsdflkjsd"+ random.nextLong());
        history.setId(new Long(random.nextLong()));
        return history;
    }

    private static Policy getPolicy()
    {
        Policy policy = new Policy();
        policy.setCustomerId(new Long(random.nextInt()));
        policy.setId(new Long(random.nextLong()));
        return policy;
    }

    private static Claim getClaim()
    {
        Claim claim = new Claim();
        claim.setClaimAmount(new Double(random.nextDouble()));
        claim.setClaimStatus(new Integer(0));
        claim.setClaimType(new Integer(0));
        claim.setId(new Long(random.nextLong()));
        claim.setPolicyId(new Long(random.nextLong()));
        return claim;
    }


    private static CustomerHistory getCustomerHistory()
    {

        CustomerHistory history = new CustomerHistory();

        history.setFirstName("sdfkljsldfkjs dlkfj sldkfj sdf"+ random.nextLong());
        history.setGender(new Integer(0));
        history.setId(new Long(random.nextLong()));
        history.setLastName("sdfkljsldfkjsdflkjsdflkj"+ random.nextLong());
        history.setPhoneNumber(new Integer(random.nextInt()));
        history.setStatus(new Integer(50));

        return history;
    }

    private static Customer getCustomer()
    {
        Customer customer = new Customer();
        customer.setFirstName("a" + random.nextLong());
        customer.setGender(new Integer(0));
        customer.setId(new Long(1));
        customer.setLastName("b" + random.nextLong());
        customer.setPhoneNumber(new Integer(0));
        customer.setPolicyId(new Long(0));
        customer.setStatus(new Integer(0));

        return customer;
    }

    private static Address getAddress()
    {
        Address address = new Address();
        address.setAddressLine1("sdflkjsdflkjsdflkjsdf"+ random.nextLong());
        address.setAddressLine2("sdfkljsldfkjskldfjsldfkj"+ random.nextLong());
        address.setAddressLine3("sdflkjsdflkjsfdlkjsdf"+ random.nextLong());
        address.setId(new Long(random.nextLong()));
        return address;
    }

}
