/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

package org.jboss.serial.data;

import java.io.Serializable;
import java.util.Random;

/**
 * $Id: TestDomainObject.java 231 2006-04-24 23:49:41Z csuconic $
 *
 * Reference for this class:
 * http://www.jboss.com/index.html?module=bb&op=viewtopic&t=71200
 *
 * From user irvingd on that forum
 *
 * @author <a href="mailto:clebert.suconic@jboss.com">Clebert Suconic</a>
 */
public class TestDomainObject implements Serializable {
	private static final long serialVersionUID = -4114310546629406876L;

	static Random random = new Random();

    public static TestDomainObject createTestInstance()
    {
       TestDomainObject testObject = new TestDomainObject(true);
       return testObject;
    }

    private static final int CHILD_COUNT = 10;

    private int originalHash;
    private TestDomainObject[] children;
    private long aLong = random.nextLong();
    private String aString = "test"  + random.nextDouble();
    private int anInt= random.nextInt();
    private int id =random.nextInt();
    private Long objLong = new Long(random.nextLong());
    private Boolean newBoolean = new Boolean(true);
    public Object aNullReference;

    TestDomainObject() {
      this (true);
      throw new RuntimeException("this constructor should never be called");
    }

    TestDomainObject(boolean hasChildren) {
      this.originalHash = System.identityHashCode(this);
      this.id = random.nextInt();

      if (hasChildren) {
        children = new TestDomainObject[CHILD_COUNT];
        for (int i=0; i<CHILD_COUNT; ++i) {
          children[i] = new TestDomainObject(false);
        }
      }
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        final TestDomainObject that = (TestDomainObject) o;

        if (aLong != that.aLong) return false;
        if (anInt != that.anInt) return false;
        if (id != that.id) return false;
        if (aString != null ? !aString.equals(that.aString) : that.aString != null) return false;

        if (aNullReference!=that.aNullReference) return false;
        
        if (aNullReference!=null || that.aNullReference!=null)
        {
        	throw new RuntimeException("aNullReference field should be Null (of course) and this wasn't respected");
        }
        
        return true;
    }
    
    public String toString()
    {
    	return "originalHashCode=" + originalHash + " and current is " + System.identityHashCode(this);
    }

    public int hashCode() {
        int result;
        result = (int) (aLong ^ (aLong >>> 32));
        result = 29 * result + (aString != null ? aString.hashCode() : 0);
        result = 29 * result + anInt;
        result = 29 * result + id;
        return result;
    }

	public TestDomainObject[] getChildren() {
		return children;
	}

	public void setChildren(TestDomainObject[] children) {
		this.children = children;
	}
    
    
}
