/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

package org.jboss.serial.invocationtest;

import java.io.Serializable;
import java.util.Random;

import org.jboss.invocation.Invocation;
import org.jboss.invocation.InvocationContext;
import org.jboss.invocation.InvocationKey;
import org.jboss.invocation.MarshalledInvocation;
import org.jboss.invocation.PayloadKey;

public class TestInvocation implements Serializable
{
	private static final long serialVersionUID = 3049140179377045637L;
	int key;

	public TestInvocation(String serializationType, Object value)
	{
		InvocationContext context = new InvocationContext();
		context.setValue("SERIALIZATION_TYPE",serializationType);
		Invocation invocation = new Invocation();
		
		// Contextual information for the interceptors
		invocation.setInvocationContext(context);
		invocation.setObjectName("This is a test only");
		invocation.setMethod(TestInvocation.class.getMethods()[0]);
		invocation.setArguments(new Object[]{value});
		  
		invocation.setValue(InvocationKey.INVOKER_PROXY_BINDING,
		                      context.getInvokerProxyBinding(),
		                      PayloadKey.AS_IS);
		
		this.invocation = new MarshalledInvocation(invocation);
		key = new Random().nextInt();
	}
	
	MarshalledInvocation invocation;
	public static TestInvocation createTestInstance(String serializationType, Object dataObject)
	{
		TestInvocation newInstance = new TestInvocation(serializationType,dataObject);
		return newInstance;
	}
	
	public boolean equals(Object obj)
	{
		return key == ((TestInvocation)obj).key;
	}
	
}
