/*
 * * JBoss, Home of Professional Open Source
 * * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * * by the @authors tag. See the copyright.txt in the distribution for a
 * * full listing of individual contributors.
 * *
 * * This is free software; you can redistribute it and/or modify it
 * * under the terms of the GNU Lesser General Public License as
 * * published by the Free Software Foundation; either version 2.1 of
 * * the License, or (at your option) any later version.
 * *
 * * This software is distributed in the hope that it will be useful,
 * * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * * Lesser General Public License for more details.
 * *
 * * You should have received a copy of the GNU Lesser General Public
 * * License along with this software; if not, write to the Free
 * * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 * */

package org.jboss.serial.regression.jbser128;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectStreamField;
import java.io.Serializable;
import java.lang.reflect.Modifier;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.EnumSet;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import javax.swing.UIDefaults;

import org.jboss.serial.io.JBossObjectInputStream;
import org.jboss.serial.io.JBossObjectOutputStream;
import org.jboss.serial.objectmetamodel.DataContainer;

import junit.framework.TestCase;

// https://issues.jboss.org/browse/JBPAPP-10696

// REQUIRES JDK 6 & JDK 7 to fully test
/*
 * @author bmaxwell
 * @author csuconic
 */
public class CommonObjectSerializationUnitTest extends TestCase
{
   protected void setUp() throws Exception
   {
      System.setProperty("org.jboss.serial.sync_binary_formats", "true");
   }
	
   protected void tearDown() throws Exception
   {
      System.setProperty("org.jboss.serial.sync_binary_formats", "false");
   }

   public void testVector() throws Exception
   {
      Vector vx = new Vector();
      //vx.add(new Date());

      test1(vx);
   }
    

   public void testBigDecimal() throws Exception
   {
      BigDecimal value = new BigDecimal(5);
      //vx.add(new Date());

      test1(value);
   }
   

   public void testUIDefaults() throws Exception
   {
      UIDefaults value = new UIDefaults();
      value.put("whatever", "don'tcare");
      //vx.add(new Date());

      test1(value);
   }
   

   public void testSDF() throws Exception
   {
      DateFormat value = SimpleDateFormat.getDateInstance();
      test1(value);
   }
   
   public void testLocale() throws Exception
   {
	  Locale value = Locale.US;
      test1(value);
   }
   
   public void testBigInteger() throws Exception
   {
      BigInteger value = new BigInteger("5");
      //vx.add(new Date());

      test1(value);
   }

   public void testSerializingCommonObjects() throws Exception
   {
 
    	List testClasses = new ArrayList();
    	List testObjects = new ArrayList();
		  List results = new ArrayList();
    	
		// java.lang.*
		testObjects.add(new java.lang.Throwable());
		testObjects.add(new java.lang.Exception());
		testObjects.add(new java.util.concurrent.atomic.AtomicInteger(1));
		testObjects.add(new java.util.concurrent.atomic.AtomicLong(1L));
		testObjects.add(new java.math.BigDecimal("12345.6789"));
		testObjects.add(new java.math.BigInteger("123456789"));
		testObjects.add(new java.lang.Byte(Byte.MAX_VALUE));
		testObjects.add(new java.lang.Boolean(true));
		testObjects.add(new java.lang.Character('B'));
		testObjects.add(this.getClass()); // java.lang.Class
		testObjects.add(new java.lang.Double(Double.MAX_VALUE));
		testObjects.add(new java.lang.Float(Float.MAX_VALUE));
		testObjects.add(new java.lang.Integer(Integer.MAX_VALUE));
		testObjects.add(new java.lang.Long(Long.MAX_VALUE));
		testObjects.add(new java.lang.Short(Short.MAX_VALUE));
		testObjects.add(new java.lang.String("Test"));
		
		// java.net.*
		try {
			// java.net.URL
			testObjects.add(new java.net.URL("http://jboss.org"));
			// java.net.URI
			testObjects.add(new java.net.URL("http://jboss.org").toURI());
		} catch(Exception e) {}
		
		// java.io.*
    	// java.io.File 
    	testObjects.add(new java.io.File("/tmp/"));

    	// javax.naming
    	testObjects.add(new javax.naming.NameClassPair("NameClassPair", "javax.naming.NameClassPair"));
    	testObjects.add(new javax.naming.directory.SearchControls());
    	
    	// javax.management
    	try {
    		testObjects.add(new javax.management.ObjectName("jboss:test=mytest"));
    	} catch(Exception e) {
    		e.printStackTrace();
    	}
    	
    	// java.util.Set
		  testClasses.add(java.util.concurrent.CopyOnWriteArraySet.class);
		  testClasses.add(java.util.EnumSet.class);		
		  testClasses.add(java.util.HashSet.class);
		  testClasses.add(java.util.LinkedHashSet.class);
		  testClasses.add(java.util.TreeSet.class);

    	// java.util.List
		  testClasses.add(java.util.AbstractList.class);
		  testClasses.add(java.util.AbstractSequentialList.class);
		  testClasses.add(java.util.ArrayList.class);
		  testClasses.add(javax.management.AttributeList.class);
		  testClasses.add(java.util.concurrent.CopyOnWriteArrayList.class);
		  testClasses.add(java.util.LinkedList.class);
		  testClasses.add(javax.management.relation.RoleList.class);
		  testClasses.add(javax.management.relation.RoleUnresolvedList.class);
		  testClasses.add(java.util.Stack.class);
		  testClasses.add(java.util.Vector.class);

    	// java.util.Map
		  testClasses.add(java.util.AbstractMap.class);
		  testClasses.add(java.security.AuthProvider.class);
		  testClasses.add(java.util.concurrent.ConcurrentHashMap.class);	
		  testClasses.add(java.util.HashMap.class);
		  testClasses.add(java.util.Hashtable.class);
		  testClasses.add(java.util.IdentityHashMap.class);
		  testClasses.add(java.util.LinkedHashMap.class);
		  testClasses.add(java.util.Properties.class);
		  testClasses.add(java.security.Provider.class);
		  testClasses.add(java.util.TreeMap.class);
		  testClasses.add(javax.swing.UIDefaults.class);
    	
      Iterator it; 
    	try 
    	{
        it = testObjects.iterator();
    		while(it.hasNext()) 
    		{
    			results.add(test((Serializable)it.next()));
    		}
        it = testClasses.iterator();
    		while(it.hasNext())
    		{
    			results.add(test((Class)it.next()));
    		}
    	}
    	catch(Exception e)
    	{
    		System.out.println("Test Failed");
    		e.printStackTrace();
        fail();
    	}    
      it = results.iterator();
    	while(it.hasNext())
		  {
				 String result = (String) it.next();
			   System.out.println(result);
				 if(result.contains("[FAILED]"))
           fail("This object failed: " + result);
		  }
    	System.out.println("Test done");
    }
    
    private static Serializable newInstance(Class clazz) throws Exception {
    	// we can't test an interface
    	if(clazz.isInterface() || Modifier.isAbstract(clazz.getModifiers()))
    		return null;
    	
    	Object object = clazz.newInstance();
    	if(object instanceof Collection) {
    		Collection collection = (Collection) object;
    		collection.add(new Date());	
    	}
    	else if(object instanceof Map) 
    	{
    		((Map)object).put("Test", new Date());
    	}
    	else if(object instanceof Set) 
    	{
    		((Set)object).add(new Date());
    	}
    	else {
    		System.out.println(clazz + " is not a collection, map or set");
    		System.out.flush();
    		System.exit(1);
    	}
    	return (Serializable) object;
    }
    
    public static String test(Class clazz) throws Exception {
    	Object object = newInstance(clazz);
    	
    	if(object == null)
    		return "[UNKNOWN] " +  clazz.getName() + ": not tested because it is abstract or an interface";
    	return test((Serializable) object);
    }
    
    public static String test(Serializable serializableObject) throws Exception {    	
    	
    	if(serializableObject == null)
    		return null;
    	String status = "[PASSED] ";
    	StringBuilder results = new StringBuilder(serializableObject.getClass().getName() + ": ");    	
    	
    	try {
    		test1(serializableObject);
    		results.append("test1: pass, ");
    	} catch(Exception e) {
    		e.printStackTrace();
    		results.append("test1: fail, ");
    		status = "[FAILED] ";
    	}    
    	try {
    		test2(serializableObject);
    		results.append("test2: pass, ");
    	} catch(Exception e) {
    		e.printStackTrace();    		
    		results.append("test2: fail, ");
    		status = "[FAILED] ";
    	}
    	try {
    		test3(serializableObject);
    		results.append("test3: pass");
    	} catch(Exception e) {
    		e.printStackTrace();
    		results.append("test3: fail");
    		status = "[FAILED] ";
    	}
    	
    	results.insert(0, status);
    	
    	return results.toString();
    }
    
    public static void test1(Serializable serializableObject) throws Exception {
        DataContainer dataContainer = new DataContainer(false);
        
        dataContainer.getOutput().writeObject(serializableObject);
        dataContainer.flush();

        Object readObject = dataContainer.getInput().readObject();
        System.out.println(readObject);
    }
    
    public static void test2(Serializable serializableObject) throws Exception {
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        JBossObjectOutputStream jBossObjectOutputStream = new JBossObjectOutputStream(output);
                    
        jBossObjectOutputStream.writeObject(serializableObject);
        jBossObjectOutputStream.flush();
        
        Object readObject = (new JBossObjectInputStream(new ByteArrayInputStream(output.toByteArray())).readObject());

        assertNotNull(readObject);
        System.out.println(readObject);
    }
    
    public static void test3(Serializable serializableObject) throws Exception {       
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        ObjectOutputStream objectOutputStream = new ObjectOutputStream(output);
        
        objectOutputStream.writeObject(serializableObject);

        objectOutputStream.flush();
        
        Object readObject = (new ObjectInputStream(new ByteArrayInputStream(output.toByteArray())).readObject());
        
        System.out.println(readObject);
    }
    
    
    public void testUnbalancedJava() throws Exception
    {
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        ObjectOutputStream objectOutputStream = new ObjectOutputStream(output);
        
        objectOutputStream.writeObject(new UnbalancedWrite(3,4));

        objectOutputStream.flush();
        
        UnbalancedWrite readObject = (UnbalancedWrite)(new ObjectInputStream(new ByteArrayInputStream(output.toByteArray())).readObject());
        
        System.out.println(readObject);
    }
    
    // Unbalanced meaning only writeObject defined
    public void testUnbalancedJBoss() throws Exception
    {
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        JBossObjectOutputStream objectOutputStream = new JBossObjectOutputStream(output);
        
        Object obj = new UnbalancedWrite(3,4);
        objectOutputStream.writeObject(obj);

        objectOutputStream.flush();
        
        UnbalancedWrite readObject = (UnbalancedWrite)(new JBossObjectInputStream(new ByteArrayInputStream(output.toByteArray())).readObject());
        
        assertEquals(3, readObject.i);
        assertEquals(0, readObject.j);

        System.out.println(readObject);
    }
    
    // Unbalanced meaning only writeObject defined
    public void testBalancedJBoss() throws Exception
    {
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        JBossObjectOutputStream objectOutputStream = new JBossObjectOutputStream(output);
        
        Object obj = new BalancedWrite(3,4);
        objectOutputStream.writeObject(obj);

        objectOutputStream.flush();
        
        BalancedWrite readObject = (BalancedWrite)(new JBossObjectInputStream(new ByteArrayInputStream(output.toByteArray())).readObject());
        
        assertEquals(3, readObject.i);
        assertEquals(0, readObject.j);

        System.out.println(readObject);
    }
    
    public void testUnbalancedJBossInvalid() throws Exception
    {
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        JBossObjectOutputStream objectOutputStream = new JBossObjectOutputStream(output);
        
        UnbalancedWrite obj = new UnbalancedWrite(3,4);
        boolean error = false;
        // force an invalid operation
        obj.playInvalid = true;
        try
        {
        	objectOutputStream.writeObject(obj);
        }
        catch (Exception e)
        {
        	error = true;
        }
        
        assertTrue(error);
    }
    
    
    public void testSomeString() throws Exception
    {
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        JBossObjectOutputStream objectOutputStream = new JBossObjectOutputStream(output);
        
        StringFieldWrite obj = new StringFieldWrite();
    	  objectOutputStream.writeObject(obj);
    	
       	StringFieldWrite readObject = (StringFieldWrite)(new JBossObjectInputStream(new ByteArrayInputStream(output.toByteArray())).readObject());
    	
    	  assertEquals("SomeString", readObject.someString);
    }
    
    public void testSomeStringRegular() throws Exception
    {
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        JBossObjectOutputStream objectOutputStream = new JBossObjectOutputStream(output);
        
        StringRegularWrite obj = new StringRegularWrite();
    	  objectOutputStream.writeObject(obj);
    	
    	  StringRegularWrite readObject = (StringRegularWrite)(new JBossObjectInputStream(new ByteArrayInputStream(output.toByteArray())).readObject());
    	
    	  assertEquals("SomeString", readObject.someString);
    }
    
    
    static class SuperClass implements Serializable
    {
    	int m;
    	
    	SuperClass(int m)
    	{
    		this.m = m;
    	}
    }
    
    static class UnbalancedWrite extends SuperClass
    {
    	public int i;
    	public int j;
    	
    	public boolean playInvalid;
    	
    	public String toString()
    	{
    		return "i=" + i + ", j=" + j + ", m=" + m;
    	}
    	
    	public UnbalancedWrite(int i, int j)
    	{
    		super(i * 2);
    		this.i = i;
    		this.j = j;
    	}
    	
    	
    	
        private void writeObject(java.io.ObjectOutputStream s)
                throws java.io.IOException {
            final java.io.ObjectOutputStream.PutField fields = s.putFields();
            fields.put("i", i);
            if (playInvalid)
            {
            	// this is suppsoed to throw an exception
            	fields.put("m", m);
            }
            s.writeFields();
        }
    	
    }
    
    
    static class StringFieldWrite implements Serializable
    {
        private static final ObjectStreamField[] serialPersistentFields = {
            new ObjectStreamField("language", String.class)
        };
        
        public volatile String someString;
        
        private void writeObject(ObjectOutputStream out) throws IOException {
            ObjectOutputStream.PutField fields = out.putFields();
            fields.put("language", "SomeString");
            out.writeFields();
        }

        private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
            ObjectInputStream.GetField fields = in.readFields();
            this.someString = (String)fields.get("language", "");
        }

    }
    
    static class StringRegularWrite implements Serializable
    {
    	public StringRegularWrite()
    	{
    		this.someString = "SomeString";
    	}
        public String someString;
    }
    
    
    static class BalancedWrite extends SuperClass
    {
    	public int i;
    	public int j;
    	
    	public boolean playInvalid;
    	
    	public String toString()
    	{
    		return "i=" + i + ", j=" + j + ", m=" + m;
    	}
    	
    	public BalancedWrite(int i, int j)
    	{
    		super(i * 2);
    		this.i = i;
    		this.j = j;
    	}
    	
    	
        private void readObject(java.io.ObjectInputStream s)
                throws java.io.IOException, ClassNotFoundException {

                ObjectInputStream.GetField fields = s.readFields();

                i = fields.get("i", -1);
        }
    	
        private void writeObject(java.io.ObjectOutputStream s)
                throws java.io.IOException {
            final java.io.ObjectOutputStream.PutField fields = s.putFields();
            fields.put("i", i);
            if (playInvalid)
            {
            	// this is suppsoed to throw an exception
            	fields.put("m", m);
            }
            s.writeFields();
        }
    	
    }

}
